﻿Imports System.Windows
Imports System.Windows.Data

Namespace MsdnDemo.Converters
    ''' <summary>
    ''' Null to visibility converter
    ''' </summary>
    Public Class NullToVisibilityConverter
        Implements IValueConverter

        ''' <summary>
        ''' Modifies the source data before passing it to the target for display in the UI.
        ''' </summary>
        ''' <param name="value">The source data being passed to the target.</param>
        ''' <param name="targetType">The <see cref="T:System.Type"/> of data expected by the target dependency property.</param>
        ''' <param name="parameter">An optional parameter to be used in the converter logic.</param>
        ''' <param name="culture">The culture of the conversion.</param>
        ''' <returns>
        ''' The value to be passed to the target dependency property.
        ''' </returns>
        Public Function Convert(ByVal value As Object, ByVal targetType As Type, ByVal parameter As Object, ByVal culture As Globalization.CultureInfo) As Object Implements IValueConverter.Convert
            If parameter IsNot Nothing AndAlso parameter.ToString() = "Inverse" Then
                Return If(value Is Nothing OrElse value.ToString() = "", Visibility.Visible, Visibility.Collapsed)
            End If

            Return If(value Is Nothing OrElse value.ToString() = "", Visibility.Collapsed, Visibility.Visible)

        End Function

        ''' <summary>
        ''' Modifies the target data before passing it to the source object.  This method is called only in <see cref="F:System.Windows.Data.BindingMode.TwoWay"/> bindings.
        ''' </summary>
        ''' <param name="value">The target data being passed to the source.</param>
        ''' <param name="targetType">The <see cref="T:System.Type"/> of data expected by the source object.</param>
        ''' <param name="parameter">An optional parameter to be used in the converter logic.</param>
        ''' <param name="culture">The culture of the conversion.</param>
        ''' <returns>
        ''' The value to be passed to the source object.
        ''' </returns>
        Public Function ConvertBack(ByVal value As Object, ByVal targetType As Type, ByVal parameter As Object, ByVal culture As Globalization.CultureInfo) As Object Implements IValueConverter.ConvertBack
            Return Nothing
        End Function
    End Class
End Namespace