﻿Imports System.Windows.Data
Imports System.Text

Namespace MsdnDemo.Converters
    ''' <summary>
    ''' Output asterisk converter
    ''' </summary>
    Public Class OutputAsteriskConverter
        Implements IValueConverter
        ''' <summary>
        ''' For databinding purposes
        ''' </summary>
        Public Shared Instance As New OutputAsteriskConverter()

        ''' <summary>
        ''' Modifies the source data before passing it to the target for display in the UI.
        ''' </summary>
        ''' <param name="value">The source data being passed to the target.</param>
        ''' <param name="targetType">The <see cref="T:System.Type"/> of data expected by the target dependency property.</param>
        ''' <param name="parameter">An optional parameter to be used in the converter logic.</param>
        ''' <param name="culture">The culture of the conversion.</param>
        ''' <returns>
        ''' The value to be passed to the target dependency property.
        ''' </returns>
        Public Function Convert(ByVal value As Object, ByVal targetType As Type, ByVal parameter As Object, ByVal culture As Globalization.CultureInfo) As Object Implements IValueConverter.Convert
            If value Is Nothing Then
                Return value
            End If

            Dim len = value.ToString().Length
            Dim returnValue = New StringBuilder("")
            For i = 0 To len - 1
                returnValue.Append("*")
            Next i

            Return returnValue.ToString()
        End Function

        ''' <summary>
        ''' Modifies the target data before passing it to the source object.  This method is called only in <see cref="F:System.Windows.Data.BindingMode.TwoWay"/> bindings.
        ''' </summary>
        ''' <param name="value">The target data being passed to the source.</param>
        ''' <param name="targetType">The <see cref="T:System.Type"/> of data expected by the source object.</param>
        ''' <param name="parameter">An optional parameter to be used in the converter logic.</param>
        ''' <param name="culture">The culture of the conversion.</param>
        ''' <returns>
        ''' The value to be passed to the source object.
        ''' </returns>
        Public Function ConvertBack(ByVal value As Object, ByVal targetType As Type, ByVal parameter As Object, ByVal culture As Globalization.CultureInfo) As Object Implements IValueConverter.ConvertBack
            Return value
        End Function
    End Class
End Namespace