﻿Imports System.Collections.Generic
Imports MsdnDemo.Infrastructure.Portable.Constants
Imports MsdnDemo.Infrastructure.Portable.Entities
Imports MsdnDemo.Infrastructure.Portable.Events
Imports MsdnDemo.LayerBusinessLogic.Base
Imports MsdnDemo.LayerBusinessLogic.Interfaces

Namespace MsdnDemo.LayerBusinessLogic
    Public Class RoleBll
        Inherits BllBase(Of RoleEntity)
        Implements IRolesBll
        Private _isProcessing As Boolean


        ''' <summary>
        ''' When a List() is requested from the Bll it will end up
        ''' here so that we can test to see if there are any roles
        ''' configured.  If not we'll add the three default roles
        ''' </summary>
        ''' <param name="e"></param>
        Protected Overrides Sub ListDataArrivedHandler(ByVal e As DalEventArgs)
            MyBase.ListDataArrivedHandler(e) ' Logs process

            ' We don't want to end up here recursively
            If _isProcessing Then
                Return
            End If

            Try
                _isProcessing = True ' Set processing flag

                Dim listDemo = New List(Of RoleEntity)(e.GetResults(Of IEnumerable(Of RoleEntity))())

                ' if there are no roles then create default roles
                If listDemo.Count = 0 Then
                    Dim status = CreateDefaultRoles(e)

                    ' If create was successful execute list
                    ' again so that it can populate e.Results
                    If status = StatusCode.Success Then
                        List(e) ' Read list
                    End If
                End If
            Finally
                _isProcessing = False ' reset processing flag
            End Try
        End Sub

        Private Function CreateDefaultRoles(ByVal e As DalEventArgs) As Integer
            Dim list = New List(Of RoleEntity) From {
             New RoleEntity With {.Id = 1, .Role = "Admin", .Description = "Adminstrator role"},
             New RoleEntity With {.Id = 2, .Role = "User", .Description = "Users role"},
             New RoleEntity With {.Id = 3, .Role = "Guest", .Description = "Guest role"}}

            ' Iterate through defaults and create records
            For Each record In list
                Dim args = New DalEventArgs With {.PrimaryKey = record.Id, .Data = record}

                ' If process fails then return status
                Dim statusDemo = Create(args)
                If statusDemo <> StatusCode.Success Then
                    Return statusDemo
                End If
            Next record
            Return StatusCode.Success
        End Function
    End Class
End Namespace