﻿Imports Microsoft.VisualBasic
Imports System.Linq
Imports System.Text
Imports System
Imports MsdnDemo.Infrastructure.Portable.Attributes
Imports MsdnDemo.Infrastructure.Portable.Events

Namespace MsdnDemo.LayerDataAccess.Sqlite
    ''' <summary>
    ''' Sql utility extensions
    ''' </summary>
    Public Module SqlUtilityExtension
        ''' <summary>
        ''' Makes the SQL insert table string.
        ''' </summary>
        ''' <param name="helper">The helper.</param>
        ''' <param name="e">The see cref="System.EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        <Runtime.CompilerServices.Extension()> _
        Public Function MakeSqlInsertTableString(ByVal helper As SqlUtility, ByVal e As EventArgs) As String
            Dim args = TryCast(e, DalEventArgs)
            If args Is Nothing Then
                Throw New NotSupportedException(String.Format("{0} is not supported", e.GetType().FullName))
            End If

            Dim delimiter = ""

            ' Get table name - we don't want ViewModel to be in the name
            Dim tablename = helper.Type.Name.Replace("ViewModel", "")

            Dim sb = New StringBuilder("insert into ")
            sb.AppendFormat("{0} ", tablename)

            Dim sbFieldList = New StringBuilder
            Dim sbValueList = New StringBuilder



            For Each propDemo In
                From prop In helper.Type.GetProperties()
                Let attributes = prop.GetCustomAttributes(GetType(DataFieldExcludeAttribute), True)
                Where attributes.Length <= 0
                Select prop
                Dim name = propDemo.Name
                Dim value = GetStringFromValue(propDemo.GetValue(args.Data, Nothing))

                ' If the current field name matches the primary key 
                If name.ToLower() = args.PrimaryKeyField.ToLower() Then
                    ' Only use if primary key was initialized in args
                    If args.PrimaryKey <> -1 Then
                        value = args.PrimaryKey
                    End If

                    If TypeOf value Is Int32 AndAlso CInt(Fix(value)) = 0 Then
                        Throw New NotSupportedException("DEVELOPER: You cannot have a primary key value of 0" & vbCrLf & "Something didn't get initialized")
                    End If
                End If

                sbFieldList.AppendFormat("{0} {1}", delimiter, name)
                sbValueList.AppendFormat("{0} {1}", delimiter, value)
                delimiter = ","
            Next propDemo

            sb.AppendFormat("({0}) values ({1})", sbFieldList, sbValueList)

            Return sb.ToString()
        End Function


        ''' <summary>
        ''' Makes the SQL update table string.
        ''' </summary>
        ''' <param name="helper">The helper.</param>
        ''' <param name="e">The see cref="System.EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        <Runtime.CompilerServices.Extension()> _
        Public Function MakeSqlUpdateTableString(ByVal helper As SqlUtility, ByVal e As EventArgs) As String
            Dim args = TryCast(e, DalEventArgs)
            If args Is Nothing Then
                Throw New NotSupportedException(String.Format("{0} is not supported", e.GetType().FullName))
            End If

            Dim delimiter = ""

            ' Get table name - we don't want ViewModel to be in the name
            Dim tablename = helper.Type.Name.Replace("ViewModel", "")

            Dim sb = New StringBuilder("update ")
            sb.AppendFormat("{0} set ", tablename)

            For Each propDemo In (
                From prop In helper.Type.GetProperties()
                Let attributes = prop.GetCustomAttributes(GetType(DataFieldExcludeAttribute), True)
                Where attributes.Length <= 0
                Select prop).Where(Function(prop) prop.Name.ToLower() <> "id")
                sb.AppendFormat("{0} {1}={2}", delimiter, propDemo.Name, GetStringFromValue(propDemo.GetValue(args.Data, Nothing)))

                delimiter = ","
            Next propDemo

            sb.AppendFormat(" where {0}={1}", args.PrimaryKeyField, args.PrimaryKey)
            Return sb.ToString()
        End Function

        Private Function GetStringFromValue(ByVal data As Object) As Object
            If TypeOf data Is Int32 Then
                Return data
            End If
            Return String.Format("'{0}'", data)
        End Function

        ''' <summary>
        ''' Makes the SQL create table string.
        ''' </summary>
        ''' <param name="helper">The helper.</param>
        ''' <returns></returns>
        <Runtime.CompilerServices.Extension()> _
        Public Function MakeSqlCreateTableString(ByVal helper As SqlUtility) As String
            Dim delimiter = ""

            Dim sb = New StringBuilder("create table ")

            ' We don't want ViewModel to be in the name
            sb.AppendFormat("{0} (", helper.Type.Name.Replace("ViewModel", ""))

            For Each propDemo In
                From prop In helper.Type.GetProperties()
                Let attributes = prop.GetCustomAttributes(GetType(DataFieldExcludeAttribute), True)
                Where attributes.Length <= 0
                Select prop
                sb.AppendFormat("{0} {1}", delimiter, Field(propDemo.Name, propDemo.PropertyType.Name))

                delimiter = ","
            Next propDemo

            sb.AppendFormat(" )")

            Return sb.ToString()
        End Function

        Private Function Field(ByVal fieldName As String, ByVal typeName As String) As String
            If fieldName.ToLower().Trim().Equals("id") Then
                Return "id int primary key"
            End If

            Return String.Format("{0} {1}", fieldName, Parse(typeName))
        End Function

        Private Function Parse(ByVal typeName As String) As String
            If typeName.ToLower().Contains("int") Then
                Return "int"
            End If

            Return "text"
        End Function

    End Module
End Namespace