﻿Imports System.Windows.Controls
Imports Microsoft.Practices.Unity
Imports MsdnDemo.Base
Imports MsdnDemo.Infrastructure.Portable.Constants
Imports MsdnDemo.Infrastructure.Portable.Entities
Imports MsdnDemo.Infrastructure.Portable.Events
Imports MsInterfaces = MsdnDemo.Infrastructure.Portable.Interfaces
Imports MsdnDemo.Infrastructure.Portable.Types
Imports MsdnDemo.LayerBusinessLogic.Interfaces
Imports MsdnDemo.MvpVmViewModels
Imports MsdnDemo.MvpVmViews

Namespace MsdnDemo.MvpVmPresenters
    Public Class MainPresenter
        Inherits PresentationPresenterBase(Of MainView, MainViewModel)
        <Dependency()>
        Public Property SecurityViewModel() As MsInterfaces.ISecurityViewModel

        <Dependency()>
        Public Property UserBll() As IUserBll

        Protected Overrides Sub Initialize()
            MyBase.Initialize() ' Logs initialize

            ' Add the view model to the view model holder
            View.ViewModelHolder.Items.Add(ViewModel)

        End Sub

        Public Overrides Sub ViewActivated(ByVal e As ViewEventArgs)
            MyBase.ViewActivated(e) ' Log it

            ' Assign role flags as applicable
            ViewModel.IsAdmin = SecurityViewModel.IsInRole("Admin")
            ViewModel.IsGuest = SecurityViewModel.IsInRole("Guest")

            If ViewModel.IsAdmin Then
                ' Get the complete user list and update view model
                ViewModel.UserList = UserBll.GetUserList()
                ViewModel.SelectedUser = ViewModel.UserList(0)
            Else
                ' Get the user record for this user and update view model
                Dim userArgs = New DalEventArgs With {.PrimaryKey = SecurityViewModel.Id}
                Dim statusDemo = UserBll.Read(userArgs)
                If statusDemo = StatusCode.Success Then
                    ViewModel.SelectedUser = userArgs.GetResults(Of UserEntity)()
                End If
            End If
        End Sub

        Protected Overrides Sub ExecuteButtonCommandHandler(ByVal sender As Object, ByVal e As DelegateCommandEventArgs)
            MyBase.ExecuteButtonCommandHandler(sender, e) ' Log we are here

            Dim buttonName = (CType(e.Sender, Button)).Name

            Select Case buttonName
                Case "btnLogin"
                    ' Activate the login screen (SecurityView) in the SecurityModule
                    RaiseViewEvent(MvpVm.SecurityModule, MvpVm.SecurityView, ProcessType.ActivateView)

                Case "btnGuestView"
                    ' Activate the login screen (SecurityView) in the SecurityModule
                    RaiseModuleEvent(MvpVm.GuestModule, MvpVm.GuestView, ModuleEventType.Login)
            End Select
        End Sub
    End Class
End Namespace