﻿Imports Microsoft.VisualBasic
Imports System.Windows.Controls
Imports Microsoft.Practices.Unity
Imports MsdnDemo.Base
Imports MsdnDemo.Infrastructure.Portable.Constants
Imports MsdnDemo.Infrastructure.Portable.Events
Imports MsdnDemo.LayerBusinessLogic.Interfaces
Imports MsdnDemo.MvpVmViewModels
Imports MsdnDemo.MvpVmViews

Namespace MsdnDemo.MvpVmPresenters
    Public Class SecurityPresenter
        Inherits PresentationPresenterBase(Of SecurityView, SecurityViewModel)
        <Dependency()>
        Public Property UserBll() As IUserBll

        Protected Overrides Sub Initialize()
            MyBase.Initialize()

            ' Add the initialized view model to the ItemsControl
            View.ViewModelHolder.Items.Add(ViewModel)
        End Sub

        ''' <summary>
        ''' Executes the button command handler.  Works with the ButtonCommand
        ''' in the superclass which is bound to in the ButtonStyle 
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.DelegateCommandEventArgs"/> instance containing the event data.</param>
        Protected Overrides Sub ExecuteButtonCommandHandler(ByVal sender As Object, ByVal e As DelegateCommandEventArgs)
            ViewModel.Message = Nothing ' reset message

            MyBase.ExecuteButtonCommandHandler(sender, e) ' Log that we are here

            Dim buttonName = (CType(e.Sender, Button)).Name

            ' Save typing for demo
            If ViewModel.UserInfo.Login IsNot Nothing AndAlso (Not ViewModel.UserInfo.Login.Contains("@mydomain.net")) Then
                ViewModel.UserInfo.Login &= "@mydomain.net"
            End If

            ' Authenticate current login using provided password
            ViewModel.IsAuthenticated = UserBll.AuthenticateUser(ViewModel.Password)

            If Not ViewModel.IsAuthenticated Then
                Dim login = ViewModel.UserInfo.Login
                If String.IsNullOrEmpty(login) OrElse String.IsNullOrEmpty(ViewModel.Password) Then
                    RaiseMessageEvent("Login/Password cannot be blank!")
                Else
                    RaiseMessageEvent(String.Format("{0} is not authenticated!" & vbCrLf & "Login and/or Password invalid", ViewModel.UserInfo.Login))
                End If
                Return
            End If

            Select Case buttonName
                Case "btnLogin"
                    ' If the user is a Guest then send them to the GuestModule/View
                    If ViewModel.IsGuest Then
                        RaiseModuleEvent(MvpVm.GuestModule, MvpVm.GuestView, ModuleEventType.Login)
                    Else
                        ' Everyone else goes to the MainModule/View
                        RaiseModuleEvent(MvpVm.MainModule, MvpVm.MainView, ModuleEventType.Login)
                    End If
            End Select
        End Sub

        Public Overrides Sub MessageEventHandler(ByVal e As MessageEventArgs)
            ' We only want to capture messages received from this
            ' presenter processing
            If TypeOf e.Sender Is SecurityPresenter Then
                ViewModel.Message = e.Message
            End If
        End Sub

    End Class
End Namespace