﻿Imports System.Collections.Generic
Imports MsdnDemo.Base
Imports MsdnDemo.Infrastructure.Portable.Entities

Namespace MsdnDemo.MvpVmViewModels
    Public Class MainViewModel
        Inherits PresentationViewModelBase
        Private _message As String
        Private _userList As List(Of UserEntity)

        Public Property Message() As String
            Get
                Return _message
            End Get
            Set(ByVal value As String)
                _message = value
                OnPropertyChanged("Message")
            End Set
        End Property


        Public Property UserList() As List(Of UserEntity)
            Get
                Return _userList
            End Get
            Set(ByVal value As List(Of UserEntity))
                _userList = value

                If value IsNot Nothing Then
                    SelectedUser = value(0)
                End If

                OnPropertyChanged("UserList")
            End Set
        End Property


#Region "lazy instantiation of _selected user if doesn't exist"
        ' We're going to cheat and use a backing field as a property
        ' so that we can perform lazy instantiation if the _selectedUser
        ' is null (preventing us from having to do it in each property)
        Private __selectedUser As UserEntity
        Private Property _selectedUser() As UserEntity
            Get
                If __selectedUser Is Nothing Then
                    __selectedUser = New UserEntity()
                End If
                Return __selectedUser
            End Get
            Set(ByVal value As UserEntity)
                __selectedUser = value
            End Set
        End Property
#End Region

        ' Our UserEntity does not implement INotifyPropertyChanged
        ' So we'll wrap it at the ViewModel level and make the
        ' applicable notification 

        Public Property SelectedUser() As UserEntity
            Get
                Return _selectedUser
            End Get
            Set(ByVal value As UserEntity)
                _selectedUser = value

                OnPropertyChanged("SelectedUser")
                OnPropertyChanged("FirstName")
                OnPropertyChanged("LastName")
                OnPropertyChanged("Password")
                OnPropertyChanged("PrimaryEmail")
            End Set
        End Property

        Public Property FirstName() As String
            Get
                Return _selectedUser.FirstName
            End Get
            Set(ByVal value As String)
                _selectedUser.FirstName = value
                OnPropertyChanged("FirstName")
            End Set
        End Property

        Public Property LastName() As String
            Get
                Return _selectedUser.LastName
            End Get
            Set(ByVal value As String)
                _selectedUser.LastName = value
                OnPropertyChanged("LastName")
            End Set
        End Property

        Public Property Password() As String
            Get
                Return _selectedUser.Password
            End Get
            Set(ByVal value As String)
                _selectedUser.Password = value
            End Set
        End Property

        Public Property PrimaryEmail() As String
            Get
                Return _selectedUser.PrimaryEmail
            End Get
            Set(ByVal value As String)
                _selectedUser.PrimaryEmail = value
                OnPropertyChanged("PrimaryEmail")
            End Set
        End Property
    End Class
End Namespace