﻿Imports System
Imports System.Windows
Imports Microsoft.Practices.Prism.Modularity
Imports Microsoft.Practices.Prism.Regions
Imports Microsoft.Practices.Prism.UnityExtensions
Imports Microsoft.Practices.Unity
Imports MsdnDemo.Infrastructure.Portable.Interfaces
Imports MsdnDemo.Infrastructure.Portable.Configuration
Imports MsdnDemo.Infrastructure.Portable.Logger
Imports MsdnDemo.Modules

Namespace MsdnDemo.Prism
    ''' <summary>
    ''' Bootstrapper
    ''' </summary>
    Public Class GwnBootstrapper
        Inherits UnityBootstrapper
        Implements IBootstrapper
        Private _shell As IShell
        Private _logger As ILogger

        Public InitializeShellCallBack As Action(Of IShell) = Sub(shell)
                                                              End Sub

        ''' <summary>
        ''' Initializes a new instance of the <see cref="GwnBootstrapper"/> class.
        ''' </summary>
        Public Sub New()
        End Sub

        ''' <summary>
        ''' Initializes a new instance of the <see cref="GwnBootstrapper"/> class.
        ''' </summary>
        ''' <param name="container">The container.</param>
        Public Sub New(ByVal container As IUnityContainer)
            Container = container
        End Sub

        ''' <summary>
        ''' Configures the <see cref="IModuleCatalog"/> used by Prism.
        ''' </summary>
        Protected Overrides Sub ConfigureModuleCatalog()
            MyBase.ConfigureModuleCatalog()

            Dim moduleCatalog = TryCast(Me.ModuleCatalog, ModuleCatalog)
            If moduleCatalog Is Nothing Then
                Throw New InvalidOperationException("Module Catalog is not available in GwnBootstrapper")
            End If

            ' The application controller will respond to the Login event (ModuleEvent) and
            ' load all of the modules configured to load on-demand
            moduleCatalog.AddModule(GetType(ApplicationControllerModule))

            ' We're going to use the default security module (in the Gwn.Infrastructure project)
            moduleCatalog.AddModule(GetType(SecurityModule))

            ' Will only be loaded on successful login
            LoadModuleOnDemand(GetType(MainModule), "User,Admin")
            LoadModuleOnDemand(GetType(GuestModule), "User,Guest")
        End Sub

        Private Sub LoadModuleOnDemand(ByVal moduleType As Type, ByVal roles As String)
            CType(ModuleCatalog, ModuleCatalog).AddModule(New ModuleInfo With {.ModuleName = moduleType.Name & ":" & roles, .ModuleType = moduleType.AssemblyQualifiedName, .InitializationMode = InitializationMode.OnDemand})
        End Sub


        ''' <summary>
        ''' Used by the Phone so that we can get a reference to the 
        ''' Shell since the MainPage.xaml.vb is the starting point
        ''' </summary>
        ''' <param name="shell">The shell.</param>
        Public Shadows Sub Run(ByVal shell As IShell) Implements IBootstrapper.Run
            _shell = shell
            Run()
        End Sub

        Public Shadows Sub Run() Implements IBootstrapper.Run
            MyBase.Run()
        End Sub

        ''' <summary>
        ''' Configures the <see cref="IUnityContainer"/>. 
        ''' May be overwritten in a derived class to add specific
        ''' type mappings required by the application.
        ''' </summary>
        Protected Overrides Sub ConfigureContainer()
            MyBase.ConfigureContainer()

            ' Global registrations
            Container.RegisterType(Of IAppConfig, AppConfig)(New ContainerControlledLifetimeManager()).RegisterType(Of ILogger, DebugLogger)()

            _logger = Container.Resolve(Of ILogger)()
            _logger.Log("{0}.ConfigureContainer()", Me.GetType())

        End Sub

        ''' <summary>
        ''' Creates the shell or main window of the application.
        ''' </summary>
        ''' <returns>The shell of the application.</returns>
        ''' <remarks>
        ''' If the returned instance is a <see cref="DependencyObject"/>, the
        ''' <see cref="GwnBootstrapper"/> will attach the default <seealso cref="IRegionManager"/> of
        ''' the application in its <see cref="RegionManager.RegionManagerProperty"/> attached property
        ''' in order to be able to add regions by using the <seealso cref="RegionManager.RegionNameProperty"/>
        ''' attached property from XAML.
        ''' </remarks>
        Protected Overrides Function CreateShell() As DependencyObject
            _logger.Log("{0}.CreateShell()", Me.GetType())

            ' _shell will not be null for Windows Phone
            If _shell Is Nothing Then
                _shell = Container.Resolve(Of MainPage)()
            End If

            ' Register an instance of the shell with container
            Container.RegisterInstance(Of IShell)(_shell)

            Return CType(_shell, DependencyObject)
        End Function

        ''' <summary>
        ''' Initializes the shell.
        ''' </summary>
        Protected Overrides Sub InitializeShell()
            ' Logger.LogDebug("CreateShell", GetType());
            MyBase.InitializeShell()

            ' Provide application a hook into Initialize
            ' shell - each platform has different responsibilities
            InitializeShellCallBack(CType(Shell, IShell))

        End Sub
    End Class
End Namespace