﻿Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Input
Imports System.Windows.Interactivity
Imports Microsoft.Practices.Prism.Interactivity
Imports MsdnDemo.Infrastructure.Portable.Events

Namespace MsdnDemo.TriggerActions
    ''' <summary>
    ''' Modified the following to emulate Desktop/Silverlight environments
    ''' http://compiledexperience.com/blog/posts/blendable-mvvm-commands-and-behaviors
    ''' Uses Prism.Interactivity.Phone 
    ''' </summary>
    Public Class ExecuteCommandAction
        Inherits TriggerAction(Of FrameworkElement)
        Private ReadOnly _commandListener As BindingListener
        Private ReadOnly _commandParameterListener As BindingListener


        ''' <summary>
        ''' Gets or sets the command.
        ''' </summary>
        ''' <value>The command.</value>
        Public Property Command() As Binding
            Get
                Return CType(GetValue(CommandProperty), Binding)
            End Get
            Set(ByVal value As Binding)
                SetValue(CommandProperty, value)
            End Set
        End Property

        ''' <summary>
        ''' Command property
        ''' </summary>
        Public Shared ReadOnly CommandProperty As DependencyProperty = DependencyProperty.Register("Command", GetType(Binding), GetType(ExecuteCommandAction), New PropertyMetadata(Nothing, Sub(d, e)
                                                                                                                                                                                                 CType(d, ExecuteCommandAction).OnCommandBindingChanged(e)
                                                                                                                                                                                             End Sub))

        ''' <summary>
        ''' Gets or sets the command parameter.
        ''' </summary>
        ''' <value>The command parameter.</value>
        Public Property CommandParameter() As Binding
            Get
                Return CType(GetValue(CommandParameterProperty), Binding)
            End Get
            Set(ByVal value As Binding)
                SetValue(CommandParameterProperty, value)
            End Set
        End Property

        ''' <summary>
        ''' CommandParameter property
        ''' </summary>
        Public Shared ReadOnly CommandParameterProperty As DependencyProperty = DependencyProperty.Register("CommandParameter", GetType(Binding), GetType(ExecuteCommandAction), New PropertyMetadata(Nothing, Sub(d, e)
                                                                                                                                                                                                                   CType(d, ExecuteCommandAction).OnCommandParameterBindingChanged(e)
                                                                                                                                                                                                               End Sub))


        ''' <summary>
        ''' Initializes a new instance of the <see cref="ExecuteCommandAction"/> class.
        ''' </summary>
        Public Sub New()
            _commandListener = New BindingListener()
            _commandParameterListener = New BindingListener()
        End Sub

        ''' <summary>
        ''' Called after the action is attached to an AssociatedObject.
        ''' </summary>
        Protected Overrides Sub OnAttached()
            MyBase.OnAttached()

            _commandListener.Element = AssociatedObject
            _commandParameterListener.Element = AssociatedObject
        End Sub

        ''' <summary>
        ''' Called when the action is being detached from its 
        ''' AssociatedObject, but before it has actually occurred.
        ''' </summary>
        Protected Overrides Sub OnDetaching()
            MyBase.OnDetaching()

            _commandListener.Element = Nothing
            _commandParameterListener.Element = Nothing
        End Sub


        ''' <summary>
        ''' Invokes the action.
        ''' </summary>
        ''' <param name="parameter">The parameter to the action. 
        ''' If the Action does not require a parameter, the parameter 
        ''' may be set to a null reference.</param>
        Protected Overrides Sub Invoke(ByVal parameter As Object)
            Dim command = TryCast(_commandListener.Value, ICommand)

            If command Is Nothing Then
                Return
            End If

            Dim args = New DelegateCommandEventArgs

            
            ' <Style TargetType="Button" x:Key="ButtonStyle"

            '            <Setter Property="Template">
            '            <Setter.Value>
            '                <ControlTemplate TargetType="Button">
            '                    <StackPanel Orientation="Horizontal">
            '                        <Border CornerRadius="20" 
            '                                    BorderBrush="{TemplateBinding BorderBrush}"
            '                                    BorderThickness="{TemplateBinding BorderThickness}"
            '                                    Padding="0" Margin="20">
            '                            <Button
            '                                    BorderThickness="0"
            '                                    Margin="{TemplateBinding Margin}"
            '                                    Height="{TemplateBinding Height}"
            '                                    Width="{TemplateBinding Width}"
            '                                    Background="{TemplateBinding Background}"
            '                                    Foreground="{TemplateBinding Foreground}"
            '                                    Tag="{Binding RelativeSource={RelativeSource TemplatedParent }}"
            '                                    >
            '                                <Custom:Interaction.Triggers>
            '                                    <Custom:EventTrigger EventName="Click" >
            '                                        <TriggerActions:ExecuteCommandAction 
            '                                                Command="{Binding ButtonCommand}" 
            '                                                CommandParameter="{Binding RelativeSource={RelativeSource TemplatedParent }}"/>
            '                                    </Custom:EventTrigger>
            '                                </Custom:Interaction.Triggers>
            '                                <Button.Content>
            '                                    <ContentPresenter/>
            '                                </Button.Content>
            '                            </Button>
            '                        </Border>
            '                    </StackPanel>
            '                </ControlTemplate>
            '            </Setter.Value>
            '            </Setter>
            '            </Style>

            ' Unlike Desktop/Silverlight we can't get the TemplatedParent button
            ' reference in our ButtonStyle, we can only get the "template" button.
            ' However the template button Tag can bind to the actual button so 
            ' we'll pull it out of the tag and update sender with it so it will 
            ' be consistent with Desktop and Silverlight

            Dim btn = TryCast(_commandListener.Element, Button)
            If btn IsNot Nothing Then
                Dim button = TryCast(btn.Tag, Button)
                If button IsNot Nothing Then ' For template
                    args.Sender = button
                Else
                    args.Sender = btn ' for default use
                End If
            Else ' otherwise pass in the element
                args.Sender = _commandListener.Element
            End If

            If command.CanExecute(args) Then
                command.Execute(args)
            End If
        End Sub

        Private Sub OnCommandBindingChanged(ByVal e As DependencyPropertyChangedEventArgs)
            _commandListener.Binding = CType(e.NewValue, Binding)
        End Sub

        Private Sub OnCommandParameterBindingChanged(ByVal e As DependencyPropertyChangedEventArgs)
            _commandParameterListener.Binding = CType(e.NewValue, Binding)
        End Sub
    End Class
End Namespace