﻿Imports System

Namespace MsdnDemo.Infrastructure.Portable.Events
    ''' <summary>
    ''' Data Access Layer Event Args
    ''' </summary>
    Public Class DalEventArgs
        Inherits EventArgs
        ''' <summary>
        ''' Initializes a new instance of the <see cref="DalEventArgs"/> class.
        ''' </summary>
        Public Sub New()
            PrimaryKey = -1
            PrimaryKeyField = "id"
            IsEventRaised = True
        End Sub

        ''' <summary>
        ''' Initializes a new instance of the <see cref="DalEventArgs"/> class.
        ''' </summary>
        ''' <param name="type">The type.</param>
        Public Sub New(ByVal type As [Enum])
            Me.New()
            Me.Type = type
        End Sub

        ''' <summary>
        ''' Gets or sets a value indicating whether this instance is event raised.
        ''' </summary>
        ''' <value>
        ''' 	<c>true</c> if this instance is event raised; otherwise, <c>false</c>.
        ''' </value>
        Public Property IsEventRaised() As Boolean


        Private _data As Object
        ''' <summary>
        ''' Gets or sets the Data.
        ''' </summary>
        ''' <value>The start date.</value>
        Public Property Data() As Object
            Get
                Return _data
            End Get
            Set(ByVal value As Object)
                PriorData = _data
                _data = value
            End Set
        End Property

        ''' <summary>
        ''' Gets or sets the results.
        ''' </summary>
        ''' <value>The results.</value>
        Public Property Results() As Object

        ''' <summary>
        ''' Gets or sets the prior data.
        ''' </summary>
        ''' <value>The prior data.</value>
        Public Property PriorData() As Object

        ''' <summary>
        ''' Gets or sets the primary key.
        ''' </summary>
        ''' <value>The primary key.</value>
        Public Property PrimaryKey() As Integer

        ''' <summary>
        ''' Gets or sets the unique key.
        ''' </summary>
        ''' <value>The unique key.</value>
        Public Property UniqueKey() As Guid

        ''' <summary>
        ''' Gets or sets the primary key field.
        ''' </summary>
        ''' <value>The primary key field.</value>
        Public Property PrimaryKeyField() As String

        ''' <summary>
        ''' Gets or sets the exception.
        ''' </summary>
        ''' <value>The exception.</value>
        Public Property Exception() As Exception

        ''' <summary>
        ''' Gets or sets the event args.
        ''' </summary>
        ''' <value>The event args.</value>
        Public Property EventArgs() As EventArgs

        ''' <summary>
        ''' Gets or sets the type.
        ''' </summary>
        ''' <value>The type.</value>
        Public Property Type() As [Enum]

        ''' <summary>
        ''' Gets or sets the where clause.
        ''' </summary>
        ''' <value>The where clause.</value>
        Public Property WhereClause() As String

        ''' <summary>
        ''' Gets the data.
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <returns></returns>
        Public Function GetData(Of T)() As T
            If TypeOf Data Is T Then
                Return CType(Data, T)
            End If

            Return Nothing

        End Function

        ''' <summary>
        ''' Gets the results.
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <returns></returns>
        Public Function GetResults(Of T)() As T
            If Results Is Nothing AndAlso TypeOf Data Is T Then
                Return CType(Data, T)
            End If

            If TypeOf Results Is T Then
                Return CType(Results, T)
            End If

            Return Nothing

        End Function

        ''' <summary>
        ''' Gets or sets a value indicating whether this instance is process.
        ''' Useful to prevent recursive processing
        ''' </summary>
        ''' <value>
        ''' 	<c>true</c> if this instance is process; otherwise, <c>false</c>.
        ''' </value>
        Public Property IsProcessing() As Boolean

        ''' <summary>
        ''' Gets or sets a value indicating whether this instance is table created.
        ''' Necessary because the data access layer will have an opportunity to
        ''' create a table prior to the business logic layer - we don't want them
        ''' both creating it
        ''' </summary>
        ''' <value>
        ''' 	<c>true</c> if this instance is table created; otherwise, <c>false</c>.
        ''' </value>
        Public Property IsTableCreated() As Boolean
    End Class
End Namespace