﻿Imports System
Imports MsdnDemo.Infrastructure.Portable.Types

Namespace MsdnDemo.Infrastructure.Portable.Events
    ''' <summary>
    ''' Data event arguments
    ''' </summary>
    Public Class DataEventArgs
        Inherits EventArgs
        ''' <summary>
        ''' Gets or sets the sender id.
        ''' </summary>
        ''' <value>The sender id.</value>
        Public Property SenderId() As Guid

        ''' <summary>
        ''' Gets or sets the sender.
        ''' </summary>
        ''' <value>The sender.</value>
        Public Property Sender() As Object

        ''' <summary>
        ''' Gets or sets the type of the process.
        ''' </summary>
        ''' <value>The type of the process.</value>
        Public Property ProcessType() As ProcessType

        ''' <summary>
        ''' Gets or sets the event args.
        ''' </summary>
        ''' <value>The event args.</value>
        Public Property EventArgs() As EventArgs

        ''' <summary>
        ''' Gets or sets a value indicating whether this instance is sucessful.
        ''' </summary>
        ''' <value>
        ''' 	<c>true</c> if this instance is sucessful; otherwise, <c>false</c>.
        ''' </value>
        Public Property IsSuccessful() As Boolean

        ''' <summary>
        ''' Gets or sets the message.
        ''' </summary>
        ''' <value>The message.</value>
        Public Property Message() As String

        ''' <summary>
        ''' Gets or sets the friendly message.
        ''' </summary>
        ''' <value>The friendly message.</value>
        Public Property FriendlyMessage() As String

        Private _exception As Exception
        ''' <summary>
        ''' Gets or sets the exception.
        ''' </summary>
        ''' <value>The exception.</value>
        Public Property Exception() As Exception
            Get
                Return _exception
            End Get
            Set(ByVal value As Exception)
                _exception = value
                IsSuccessful = False
            End Set
        End Property

        ''' <summary>
        ''' Gets or sets the call back.
        ''' </summary>
        ''' <value>The call back.</value>
        Public Property CallBack() As Action(Of DataEventArgs)

        ''' <summary>
        ''' Gets or sets the data.
        ''' </summary>
        ''' <value>The data.</value>
        Public Property Data() As Object

        ''' <summary>
        ''' Initializes a new instance of the <see cref="DataEventArgs"/> class.
        ''' </summary>
        Public Sub New()
            IsSuccessful = True
        End Sub

        ''' <summary>
        ''' Initializes a new instance of the <see cref="DataEventArgs"/> class.
        ''' </summary>
        ''' <param name="data">The data.</param>
        Public Sub New(ByVal data As Object)
            Me.New()
            Me.Data = data
        End Sub

        ''' <summary>
        ''' Gets the data.
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <returns></returns>
        Public Function GetData(Of T)() As T
            If TypeOf Data Is T Then
                Return CType(Data, T)
            End If

            Return Nothing
        End Function

        ''' <summary>
        ''' Sets the exception.
        ''' </summary>
        ''' <param name="ex">The ex.</param>
        Public Sub SetException(ByVal ex As Exception)
            Exception = ex
            IsSuccessful = False
            Message = ex.Message
        End Sub


        ''' <summary>
        ''' Gets or sets a value indicating whether this instance is handled by call back.
        ''' </summary>
        ''' <value>
        ''' 	<c>true</c> if this instance is handled by call back; otherwise, <c>false</c>.
        ''' </value>
        Public Property IsHandledByCallBack() As Boolean

        ''' <summary>
        ''' Gets or sets a value indicating whether this instance is event raised.
        ''' </summary>
        ''' <value>
        ''' 	<c>true</c> if this instance is event raised; otherwise, <c>false</c>.
        ''' </value>
        Public Property IsEventRaised() As Boolean

        ''' <summary>
        ''' Gets or sets a value indicating whether this instance is call back status.
        ''' </summary>
        ''' <value>
        ''' 	<c>true</c> if this instance is call back status; otherwise, <c>false</c>.
        ''' </value>
        Public Property IsCallBackStatus() As Boolean
    End Class
End Namespace