﻿Imports System
Imports System.Collections.Generic
Imports MsdnDemo.Infrastructure.Portable.Events

Namespace MsdnDemo.Infrastructure.Portable.Interfaces
    ''' <summary>
    ''' Data Access Layer interface
    ''' </summary>
    Public Interface IDal(Of TEntity)
        ''' <summary>
        ''' Occurs when [data arrived read event].
        ''' </summary>
        Event DataArrivedReadEvent As EventHandler(Of DataEventArgs)

        ''' <summary>
        ''' Gets or sets the connection string.
        ''' </summary>
        ''' <value>The connection string.</value>
        Property ConnectionString() As String

        ''' <summary>
        ''' Creates the table.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Function CreateTable(ByVal sender As Object, ByVal e As EventArgs) As Integer

        ''' <summary>
        ''' Drops the table.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Function DropTable(ByVal sender As Object, ByVal e As EventArgs) As Integer

        ''' <summary>
        ''' Creates the specified record.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Function Create(ByVal sender As Object, ByVal e As EventArgs) As Integer

        ''' <summary>
        ''' Reads the specified record.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Function Read(ByVal sender As Object, ByVal e As EventArgs) As TEntity

        ''' <summary>
        ''' Reads the list.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Function ReadList(ByVal sender As Object, ByVal e As EventArgs) As IEnumerable(Of TEntity)

        ''' <summary>
        ''' Updates the specified record.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Function Update(ByVal sender As Object, ByVal e As EventArgs) As Integer

        ''' <summary>
        ''' Deletes the specified record.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        ''' <returns></returns>
        Function Delete(ByVal sender As Object, ByVal e As EventArgs) As Integer

        ''' <summary>
        ''' Sets the connection string.
        ''' </summary>
        ''' <param name="connectionString">The connection string.</param>
        Sub SetConnectionString(ByVal connectionString As String)

        ''' <summary>
        ''' Table does not exist during read list.
        ''' </summary>
        ''' <param name="e">The <see cref="DalEventArgs"/> instance containing the event data.</param>
        Sub TableDoesNotExistHandler(ByVal e As DalEventArgs)

    End Interface
End Namespace