﻿
#Const SqliteTest = True
Imports System.Collections.Generic
Imports Microsoft.Practices.Unity
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports MsdnDemo.Events
Imports MsdnDemo.Infrastructure.Portable.Constants
Imports MsdnDemo.Infrastructure.Portable.Entities
Imports MsdnDemo.Infrastructure.Portable.Events
Imports MsdnDemo.Infrastructure.Portable.Interfaces
Imports MsdnDemo.Infrastructure.Portable.Types
Imports MsdnDemo.LayerBusinessLogic
Imports MsdnDemo.LayerBusinessLogic.Interfaces
Imports MsdnDemo.LayerDataAccess
Imports MsdnDemo.Mvc3.Tests.Base
Imports MsdnDemo.Mvc3.Tests.Mocks

Namespace MsdnDemo.Mvc3.Tests.Layers
    <TestClass()>
    Public Class BusinessLayerFixture
        Inherits FixtureBase
        Private _testBll As IBll
        Private _roleEntity As RoleEntity

        <TestInitialize()>
        Public Sub InitializeTest()
            ' Registers default values (normally supplied by Prism)
            Initialize()

#If SqliteTest Then
            ' Roles
            ' User
            ' User Role
            ' Mock
            Container.RegisterType(Of IRolesBll, RoleBll)().RegisterType(Of IDal(Of RoleEntity), RolesDal)().RegisterType(Of IUserBll, UserBll)().RegisterType(Of IDal(Of UserEntity), UserDal)().RegisterType(Of IUserRoleBll, UserRoleBll)().RegisterType(Of IDal(Of UserRoleEntity), UserRoleDal)().RegisterType(Of IMockBll, MockBll)().RegisterType(Of IDal(Of MockEntity), MockDal)()
#Else
			' TODO: Sql server registrations go here....
#End If

            _testBll = Container.Resolve(Of IRolesBll)()
            _testBll.Initialize()

            ' Record to create
            _roleEntity = New RoleEntity With {.Role = "Admin", .Description = "Administrator"}

        End Sub


        <TestMethod(), Owner("Layers")>
        Public Sub CanCreateUpdateReadDeleteRecord()
            ' Add to args, args.PrimaryKey will be set to 
            ' to the primary key for the new record
            Dim args = New DalEventArgs With {.Data = _roleEntity}

            ' Create the record
            Dim status = _testBll.Create(args)

            ' Assert success
            Assert.AreEqual(StatusCode.Success, status)
            Assert.AreEqual(1, args.PrimaryKey)

            ' Read the record
            Dim readArgs = New DalEventArgs With {.PrimaryKey = 1}
            status = _testBll.Read(readArgs)
            Assert.AreEqual(StatusCode.Success, status)
            Assert.AreEqual("Admin", readArgs.GetResults(Of RoleEntity)().Role)
            Assert.AreEqual("Administrator", readArgs.GetResults(Of RoleEntity)().Description)

            Dim data = TryCast(readArgs.Results, RoleEntity)
            Assert.IsNotNull(data)

            Dim primaryKey = data.Id
            data.Description = "New Description"
            status = _testBll.Update(New DalEventArgs With {.PrimaryKey = primaryKey, .Data = data})
            Assert.AreEqual(StatusCode.Success, status)

            ' Read the record - start with a clean event args
            ' so we can test the Data object upon return
            readArgs = New DalEventArgs With {.PrimaryKey = primaryKey}
            status = _testBll.Read(readArgs)
            Assert.AreEqual(StatusCode.Success, status)
            Assert.AreEqual("Admin", readArgs.GetResults(Of RoleEntity)().Role)
            Assert.AreEqual("New Description", readArgs.GetResults(Of RoleEntity)().Description)

            ' Create another record
            status = _testBll.Create(New DalEventArgs With {.Data = New RoleEntity With {.Role = "User", .Description = "User Role"}})
            Assert.AreEqual(StatusCode.Success, status)

            ' Get list
            Dim listArgs = New DalEventArgs
            status = _testBll.List(listArgs)
            Assert.AreEqual(StatusCode.Success, status)
            Assert.IsNotNull(listArgs.Results)

            Dim list = New List(Of RoleEntity)(listArgs.GetResults(Of IEnumerable(Of RoleEntity))())
            Assert.AreEqual(2, list.Count)

            ' Delete first record
            status = _testBll.Delete(New DalEventArgs With {.PrimaryKey = 1})
            Assert.AreEqual(StatusCode.Success, status)

            ' Get list
            status = _testBll.List(listArgs)
            Assert.AreEqual(StatusCode.Success, status)
            list = New List(Of RoleEntity)(listArgs.GetResults(Of IEnumerable(Of RoleEntity))())
            Assert.AreEqual(1, list.Count)
            Assert.AreEqual("User", list(0).Role)

            ' Cleanup
            _testBll.DropTable()
        End Sub

        <TestMethod(), Owner("Layers")>
        Public Sub CanCreateReadRecord()
            _testBll.DropTable()

            ' Add to args, args.PrimaryKey will be set to 
            ' to the primary key for the new record
            Dim args = New DalEventArgs With {.Data = _roleEntity}

            ' Create the record
            Dim status = _testBll.Create(args)

            ' Assert success
            Assert.AreEqual(StatusCode.Success, status)
            Assert.AreEqual(1, args.PrimaryKey)

            ' Read the record
            Dim readArgs = New DalEventArgs With {.PrimaryKey = 1}
            status = _testBll.Read(readArgs)
            Assert.AreEqual(StatusCode.Success, status)
            Assert.AreEqual("Admin", readArgs.GetResults(Of RoleEntity)().Role)
            Assert.AreEqual("Administrator", readArgs.GetResults(Of RoleEntity)().Description)

            ' Read the record
            readArgs = New DalEventArgs With {.PrimaryKey = 2}
            status = _testBll.Read(readArgs)
            Assert.AreEqual(StatusCode.NoData, status)

            ' Cleanup
            _testBll.DropTable()
        End Sub

        <TestMethod(), Owner("Layers")>
        Public Sub CanCreateRecord()
            ' Cleanup
            _testBll.DropTable()

            ' Add to args, args.PrimaryKey will be set to 
            ' to the primary key for the new record
            Dim args = New DalEventArgs With {.Data = _roleEntity}

            ' Create the record
            Dim status = _testBll.Create(args)

            ' Assert success
            Assert.AreEqual(StatusCode.Success, status)
            Assert.AreEqual(1, args.PrimaryKey)

            ' Cleanup
            _testBll.DropTable()
        End Sub

        <TestMethod(), Owner("Layers")>
        Public Sub CanDropTable()
            Dim status = _testBll.CreateTable()

            Assert.IsTrue(status = StatusCode.Success OrElse status = StatusCode.AlreadyExists)

            status = _testBll.DropTable()
            Assert.AreEqual(StatusCode.Success, status)

            status = _testBll.DropTable()
            Assert.AreEqual(StatusCode.DoesNotExist, status)

        End Sub


        ''' <summary>
        ''' Errors from dal are handled by BLL.
        ''' </summary>
        <TestMethod(), Owner("Layers")>
        Public Sub ErrorFromDalAreHandledByBll()
            _testBll.DropTable()

            Dim args As MessageEventArgs = Nothing

            ' Subscribe to message event and update
            ' args with message event arg 
            EventAggregator.GetEvent(Of MessageEvent)().Subscribe(Sub(e)
                                                                      args = e
                                                                  End Sub)

            ' Force an error
            _testBll.IsFailRequiredForUnitTest = True

            Dim status = _testBll.CreateTable()

            Assert.IsNotNull(args)
            Assert.IsTrue(args.IsError)
            Assert.AreEqual("Bogus error message", args.Exception.Message)
            Assert.AreEqual(StatusCode.TableCreateError, args.ErrorCode)
            Assert.AreEqual(ProcessType.CreateTable, CType(args.Type, ProcessType))

            status = _testBll.DropTable()
            Assert.AreEqual(StatusCode.TableCreateError, status)
        End Sub

        ''' <summary>
        ''' Tests the BLL can create table.
        ''' </summary>
        <TestMethod(), Owner("Layers")>
        Public Sub TestBllCanCreateTable()
            Dim args As MessageEventArgs = Nothing

            ' Subscribe to message event and update
            ' args with message event arg 
            EventAggregator.GetEvent(Of MessageEvent)().Subscribe(Sub(e)
                                                                      args = e
                                                                  End Sub)

            Dim status = _testBll.CreateTable()

            If args IsNot Nothing Then ' Table may already exists (from prior test)
                Assert.AreEqual(status, args.ErrorCode)
                Assert.AreEqual(StatusCode.TableCreateError, args.ErrorCode)
                Assert.AreEqual(StatusCode.AlreadyExists, args.StatusCode)
            Else
                Assert.AreEqual(StatusCode.Success, status)
            End If

            status = _testBll.DropTable()
            Assert.AreEqual(StatusCode.Success, status)
        End Sub

        <TestMethod(), Owner("Layers")>
        Public Sub CanResolveBllAndSetSecurityInfo()
            Dim securityVm = Container.Resolve(Of ISecurityViewModel)()
            securityVm.IsAuthenticated = True
            securityVm.Login = "Bill"
            securityVm.Roles = New List(Of String) From {"Admin", "User"}

            Dim bll = Container.Resolve(Of IUserBll)()
            Assert.IsNotNull(bll)

            bll.Initialize()

            Assert.IsTrue(securityVm.IsInRole("Admin"))
            Assert.IsTrue(securityVm.IsInRole("User"))
            Assert.IsFalse(securityVm.IsInRole("Guest"))


            bll.DropTable()
        End Sub

        ''' <summary>
        ''' Determines whether this instance [can create read update delete list BLL].
        ''' </summary>
        <TestMethod(), Owner("Layers")>
        Public Sub CanRegisterAndInitializeMockBll()
            Dim securityVm = Container.Resolve(Of ISecurityViewModel)()
            securityVm.IsAuthenticated = True

            Dim bll = Container.Resolve(Of IMockBll)()
            Assert.IsNotNull(bll)

            bll.Initialize()
        End Sub
    End Class
End Namespace