﻿Imports System.Linq
Imports Microsoft.Practices.Unity
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports MsdnDemo.Infrastructure.Portable.Events
Imports MsdnDemo.Infrastructure.Portable.Types
Imports MsdnDemo.Mvc3.Tests.Base
Imports MsdnDemo.Mvc3.Tests.Mocks
Imports MsdnDemo.Infrastructure.Portable.Constants
Imports MsdnDemo.Events
Imports System

Namespace MsdnDemo.Mvc3.Tests.Layers
    <TestClass()>
    Public Class DataLayerFixture
        Inherits FixtureBase
        <TestInitialize()>
        Public Sub InitializeTest()
            Initialize()
        End Sub

        ''' <summary>
        ''' Determines whether this instance [can create read update delete list dal].
        ''' </summary>
        <TestMethod(), Owner("Layers")>
        Public Sub CanCreateReadUpdateDeleteListDal()
            Dim dal = Container.Resolve(Of MockDal)()

            Try
                dal.CreateTable(Me, Nothing)
            Catch ex As Exception
                ' The BllBase will trap for errors where
                ' the table already exists and handle
                ' accordingly.  This isn't an error for unit testing
                ' since more errors (below) will let us know
                If Not ex.Message.ToLower().Contains("already exists") Then
                    Throw
                End If
            End Try

            Dim record = New MockEntity With {.Login = "Bill", .Password = "Password"}

            Dim primaryKey = dal.Create(record, New DalEventArgs With {.Type = ProcessType.DataCreate, .Data = record})

            record.Id = primaryKey

            Dim data = dal.Read(Me, New DalEventArgs With {.Type = ProcessType.DataRead, .PrimaryKey = primaryKey})

            Assert.AreEqual(primaryKey, data.Id)
            Assert.AreEqual("Bill", data.Login)
            Assert.AreEqual("Password", data.Password)

            record.Password = "NewPassword"

            Dim status = dal.Update(Me, New DalEventArgs With {.Type = ProcessType.DataUpdate, .Data = record, .PrimaryKey = record.Id})

            Assert.AreEqual(StatusCode.Success, status)

            data = dal.Read(Me, New DalEventArgs With {.Type = ProcessType.DataRead, .PrimaryKey = primaryKey})

            Assert.IsNotNull(data)
            Assert.AreEqual(primaryKey, data.Id)
            Assert.AreEqual("NewPassword", data.Password)

            Dim dataList = dal.ReadList(Me, New DalEventArgs With {.Type = ProcessType.DataList})
            Assert.IsNotNull(dataList)
            Assert.IsTrue(dataList.Count() > 0)
            Assert.IsNotNull(dataList.FirstOrDefault(Function(r) r.Id = primaryKey))

            status = dal.Delete(Me, New DalEventArgs With {.Type = ProcessType.DataDelete, .PrimaryKey = primaryKey})

            Assert.AreEqual(StatusCode.Success, status)


            data = dal.Read(Me, New DalEventArgs With {.Type = ProcessType.DataRead, .PrimaryKey = primaryKey})
            Assert.IsNull(data)

            ' cleanup
            dal.DropTable(Me, Nothing)

        End Sub
    End Class
End Namespace